<?
include "codetable/cp949table.ph";
include "codetable/ncrtable.ph";
include "codetable/uglyhan2Ncr.ph";

class convertStr {
  # Convert ksx charactors to ncr code
  # required include "codetable/ncrtable.ph";
  #
  function str2ncr($str) {
    $retStr="";
    for($i=0;$i<strlen($str);$i++) {
      if(ord($str[$i]) & 0x80) {  // MSB of 1st byte is set => 2byte char.
        $idx = hexdec(bin2hex($str[$i].$str[$i+1])) - hexdec("8141");
        $retStr .= "&#".$GLOBALS[ncrTable][$idx].";";
        $i++;
      } else $retStr .= $str[$i];
    }

    return $retStr;
  }

  # Convert ksx charactors to unicode
  # required include "codetable/ncrtable.ph";
  #
  function str2uni($str,$start='',$end='') {
    $retStr="";
    for($i=0;$i<strlen($str);$i++) {
      if(ord($str[$i]) & 0x80) {  // MSB of 1st byte is set => 2byte char.
        $idx = hexdec(bin2hex($str[$i].$str[$i+1])) - hexdec("8141");
        $retStr .= strtoupper($start.dechex($GLOBALS[ncrTable][$idx]).$end);
        $i++;
      } else $retStr .= $str[$i];
    }

    return $retStr;
  }

  # Convert charactors against arrange of ksx 1001 to ncr code
  # reuqired include "codetable/uglyhan2Ncr.ph";
  #
  function uglyHan2Ncr($str) {
    global $ncr1byte, $ncr2byte;
    $retStr="";
    for($i=0;$i<strlen($str);$i++) {
      if(ord($str[$i]) & 0x80) {  // MSB of 1st byte is set => 2byte char.
        $str1=$str[$i].$str[$i+1];
        if (preg_match("/[\xA1-\xFE]{2}/",$str1)) {  // EUC-KR character
          $retStr .= $str1; 
        } else {     // outside EUC-KR range
          $hi=ord($str1[0]);
          $low=ord($str1[1]);
          // Hangul syllable Ext. A in UHC/X-Windows-949/CP949
          // First byte : 0x81 - 0xA0, 2nd byte: 0x41-0x5A, 0x61-0x7A, 0x81-0xFE
          if (preg_match("/[\x81-\xA0][\x41-\xFE]/", $str1) ) { 
            $idx = ($hi-0x81)*0xc0 + $low-0x40; 
            $retStr .= "&#$ncr1byte[$idx];";  
          }
          // Hangul syllable Ext. B in UHC/X-Windows-949/CP949
          // First byte : 0xa1 - 0xc6, 2nd byte: 0x41-0x5A, 0x61-0x7A, 0x81-0xA0
          else if (preg_match("/[\xA1-\xC6][\x41-\xA0]/", $str1) ) { 
            $idx = ($hi-0xa1)*0x60 + $low-0x41; 
            $retStr .= "&#$ncr2byte[$idx];";  
          } else {     // invalid char. => U+FFFD (65533)
                     // try to recover from a stray/invalid octet with MSB=1
            $retStr .= $low & 0x80 ? "&#65533" : "&#65533;$str1[1]";
          }
        }
        $i++;
      } else {                   // US-ASCII
        $retStr .= $str[$i]; 
      }
    }
    return $retStr;
  }

  # Convert unicode to ksx charactors
  # required include "codetable/uni2ksx.ph";
  #
  function uni2ksx($str,$key='',$lkey='') {
    $strSize = strlen($key) + 4 + strlen($lkey);
    if($key) $key = preg_replace("/\\\|\+/","\\\\\\0",$key);
    if($lkey) $lkey = preg_replace("/\\\|\+/","\\\\\\0",$lkey);

    for($i=0;$i<strlen($str);$i++) {
      for($l=0;$l<$strSize;$l++) $value .= $str[$i+$l];
      if(preg_match("/{$key}[0-9a-z]{4}{$lkey}/i",$value)) {
        $value = strtoupper(preg_replace("/{$key}([0-9a-z]{4}){$lkey}/i","\\1",$value));
        $value = array_search(hexdec($value),$GLOBALS[cp949_2byte_ncr_table]);
        $value = dechex($GLOBALS[cp949_2byte_uni_ncr_table][$value]);
        $res = chr(hexdec($value[0].$value[1]));
        if(strlen($value) === 4) $res .= chr(hexdec($value[2].$value[3]));
        $retStr .= $res;
        $i += $strSize -1;
      } else $retStr .= $str[$i];
      unset($value);
    }

    return $retStr;
  }

  # convert UTF8
  #
  # conv 0 -> euc-kr to utf8
  # conv 1 -> utf8 to euc-kr
  # conv 2 -> utf8 to NCR
  # conv 3 -> utf8 to unicode
  #
  function convertUTF8($str,$conv=0) {
    switch($conv) {
      case '1' :
        $return = $this->ConvertUTF8($str,4);
        $return = $this->uni2ksx($return,"U+",";");
        break;
      case '2' :
        $return = $this->ConvertUTF8($str,1);
        $return = $this->uglyHan2Ncr($return);
        break;
      case '3' :
        $return = $this->ConvertUTF8($str,1);
        $return = $this->str2ncr($return);
        break;
      case '4' :
        for($i=0;$i<strlen($str);$i++) {
          if(ord($str[$i]) & 0x80) {
            $chr1 = decbin(ord($str[$i]));
            $chr2 = decbin(ord($str[$i+1]));
            $chr3 = decbin(ord($str[$i+2]));

            $chrCode1 = dechex(bindec($chr1[4].$chr1[5].$chr1[6].$chr1[7]));
            $chrCode2 = dechex(bindec($chr2[2].$chr2[3].$chr2[4].$chr2[5]));
            $chrCode3 = dechex(bindec($chr2[6].$chr2[7].$chr3[2].$chr3[3]));
            $chrCode4 = dechex(bindec($chr3[4].$chr3[5].$chr3[6].$chr3[7]));

            $return .= "U+".$chrCode1.$chrCode2.$chrCode3.$chrCode4.";";

            $i += 2;
          } else $return .= $str[$i];
        }
        break;
      default :
        $return = "";
        for($i=0;$i<strlen($str);$i++) {
          if(ord($str[$i]) & 0x80) {
            $idx = hexdec(bin2hex($str[$i].$str[$i+1])) - hexdec("8141");
            $s = dechex($GLOBALS[ncrTable][$idx]);

            $a = $this->checkLen(decbin(hexdec($s[0])),4);
            $b = $this->checkLen(decbin(hexdec($s[1])),4);
            $c = $this->checkLen(decbin(hexdec($s[2])),4);
            $d = $this->checkLen(decbin(hexdec($s[3])),4);
            
            $utf = array("1110".$a[0].$a[1].$a[2].$a[3],
                         "10".$b[0].$b[1].$b[2].$b[3].$c[0].$c[1],
                         "10".$c[2].$c[3].$d);

            $return .= chr(bindec($utf[0])).chr(bindec($utf[1])).chr(bindec($utf[2]));
            $i++;
          } else $return .= $str[$i];
        }
    }

    return $return;
  }

  function checkLen($str,$len,$chr='0') {
    $add = $len - strlen($str);
    for($i=0;$i<$add;$i++) {
      $ret .= "$chr";
    }
    $str = $ret.$str;
    return $str;
  }
}

# class 
$cv = new convertStr;

function ncrencode_lib($str,$type=0) {
  if(!$type) $ret = $GLOBALS[cv]->str2ncr($str);
  else $ret = $GLOBALS[cv]->uglyHan2Ncr($str);

  return $ret;
}

function uniencode_lib($str,$start='',$end='') {
  return $GLOBALS[cv]->str2uni($str,$start,$end);
}

function unidecode_lib($str,$to='cp949',$start='',$end='') {
  $ret = $GLOBALS[cv]->uni2ksx($str,$start,$end);
  if($to != "cp949") $ret = $GLOBALS[cv]->uglyHan2Ncr($ret);

  return $ret;
}

function utf8encode_lib($str) {
  return $GLOBALS[cv]->convertUTF8($str);
}

function utf8decode($str,$char='cp949') {
  $char = strtolower($char);
  switch($char) {
    case "cp949" :
      $ret = $GLOBALS[cv]->convertUTF8($str,1);
      break;
    case "euc-kr" :
      $ret = $GLOBALS[cv]->convertUTF8($str,2);
      break;
    case "ncr" :
      $ret = $GLOBALS[cv]->convertUTF8($str,3);
      break;
    case "unicode" :
      $ret = $GLOBALS[cv]->convertUTF8($str,4);
      break;
  }

  return $ret;
}
?>
